@@ -24,13 +24,13 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21004 $
+ * $Revision: 21293 $
  * $Author: rjongbloed $
- * $Date: 2008-09-16 07:08:56 +0000 (Tue, 16 Sep 2008) $
+ * $Date: 2008-10-12 18:24:41 -0500 (Sun, 12 Oct 2008) $
  */
 
-#ifndef __LIDPLUGINMGR_H
-#define __LIDPLUGINMGR_H
+#ifndef OPAL_LIDS_LIDPLUGINMGR_H
+#define OPAL_LIDS_LIDPLUGINMGR_H
 
 #ifdef P_USE_PRAGMA
 #pragma interface
@@ -663,4 +663,4 @@ class OpalPluginLID : public OpalLineInterfaceDevice
 };
 
 
-#endif // __LIDPLUGINMGR_H
+#endif // OPAL_LIDS_LIDPLUGINMGR_H
