@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22029 $
- * $Author: csoutheren $
- * $Date: 2009-02-10 05:25:00 +0000 (Tue, 10 Feb 2009) $
+ * $Revision: 22902 $
+ * $Author: rjongbloed $
+ * $Date: 2009-06-15 23:58:09 -0500 (Mon, 15 Jun 2009) $
  */
 
 #ifndef OPAL_SIP_SDP_H
@@ -54,6 +54,7 @@ class SDPBandwidth : public std::map<PString, unsigned>
     unsigned operator[](const PString & type) const;
     friend ostream & operator<<(ostream & out, const SDPBandwidth & bw);
     bool Parse(const PString & param);
+    void SetMin(const PString & type, unsigned value);
 };
 
 /////////////////////////////////////////////////////////
@@ -94,15 +95,16 @@ class SDPMediaFormat : public PObject
 
     const OpalMediaFormat & GetMediaFormat() const;
 
-    bool ToNormalisedOptions();
+    bool PreEncode();
+    bool PostDecode(unsigned bandwidth);
 
   protected:
-    virtual void UpdateMediaFormat() const;
+    void InitialiseMediaFormat();
 
-    mutable OpalMediaFormat mediaFormat;
-    RTP_DataFrame::PayloadTypes payloadType;
+    OpalMediaFormat mediaFormat;
 
     SDPMediaDescription & m_parent;
+    RTP_DataFrame::PayloadTypes payloadType;
     unsigned clockRate;
     PString encodingName;
     PString parameters;
@@ -131,8 +133,9 @@ class SDPMediaDescription : public PObject
       const OpalTransportAddress & address
     );
 
-    virtual void PrintOn(ostream & strm) const;
-    virtual void PrintOn(const OpalTransportAddress & commonAddr, ostream & str) const;
+    virtual bool PreEncode();
+    virtual void Encode(const OpalTransportAddress & commonAddr, ostream & str) const;
+    virtual bool PrintOn(ostream & strm, const PString & str) const;
 
     virtual bool Decode(const PStringArray & tokens);
     virtual bool Decode(char key, const PString & value);
@@ -180,8 +183,6 @@ class SDPMediaDescription : public PObject
 
     virtual void ProcessMediaOptions(SDPMediaFormat & sdpFormat, const OpalMediaFormat & mediaFormat);
 
-    virtual bool PrintOn(ostream & strm, const PString & str) const;
-
   protected:
     virtual SDPMediaFormat * FindFormat(PString & str) const;
     virtual void SetPacketTime(const PString & optionName, const PString & value);
@@ -241,6 +242,7 @@ class SDPVideoMediaDescription : public SDPRTPAVPMediaDescription
   public:
     SDPVideoMediaDescription(const OpalTransportAddress & address);
     virtual PString GetSDPMediaType() const;
+    virtual bool PreEncode();
 };
 
 /////////////////////////////////////////////////////////
@@ -266,7 +268,9 @@ class SDPSessionDescription : public PObject
   PCLASSINFO(SDPSessionDescription, PObject);
   public:
     SDPSessionDescription(
-      const OpalTransportAddress & address = OpalTransportAddress()
+      time_t sessionId,
+      unsigned version,
+      const OpalTransportAddress & address
     );
 
     void PrintOn(ostream & strm) const;
@@ -294,8 +298,8 @@ class SDPSessionDescription : public PObject
       const OpalTransportAddress & address
     );
 	
-    PINDEX GetOwnerSessionId() const { return ownerSessionId; }
-    void SetOwnerSessionId(PINDEX value) { ownerSessionId = value; }
+    time_t GetOwnerSessionId() const { return ownerSessionId; }
+    void SetOwnerSessionId(time_t value) { ownerSessionId = value; }
 
     PINDEX GetOwnerVersion() const { return ownerVersion; }
     void SetOwnerVersion(PINDEX value) { ownerVersion = value; }
@@ -308,6 +312,7 @@ class SDPSessionDescription : public PObject
 
     static const PString & ConferenceTotalBandwidthType();
     static const PString & ApplicationSpecificBandwidthType();
+    static const PString & TransportIndependentBandwidthType(); // RFC3890
 
   protected:
     void ParseOwner(const PString & str);
@@ -319,7 +324,7 @@ class SDPSessionDescription : public PObject
     PString sessionName;
 
     PString ownerUsername;
-    unsigned ownerSessionId;
+    time_t ownerSessionId;
     unsigned ownerVersion;
     OpalTransportAddress ownerAddress;
     OpalTransportAddress defaultConnectAddress;
