@@ -19,9 +19,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 20629 $
+ * $Revision: 22746 $
  * $Author: rjongbloed $
- * $Date: 2008-07-28 07:57:33 +0000 (Mon, 28 Jul 2008) $
+ * $Date: 2009-05-27 20:34:50 -0500 (Wed, 27 May 2009) $
  */
 
 #define _CRT_NONSTDC_NO_DEPRECATE 1
@@ -68,7 +68,7 @@
 
 static void * create_codec(const struct PluginCodec_Definition * codec)
 {
-  int opt = (int)codec->userData;
+  int opt = codec->userData != 0;
   struct gsm_state * context = gsm_create();
   gsm_option(context, GSM_OPT_WAV49, &opt);
   return context;
