@@ -19,9 +19,9 @@
  *
  * Contributor(s): Equivalence Pty. Ltd.
  *
- * $Revision: 21293 $
+ * $Revision: 22820 $
  * $Author: rjongbloed $
- * $Date: 2008-10-12 23:24:41 +0000 (Sun, 12 Oct 2008) $
+ * $Date: 2009-06-04 22:17:24 -0500 (Thu, 04 Jun 2009) $
  */
 
 #include "precompile.h"
@@ -55,8 +55,8 @@ void CallGen::Main()
              "f-fast-disable."
              "g-gatekeeper:"
              "I-in-dir:"
-	     "-h323-interface:"
-	     "-sip-interface:"
+             "-h323-interface:"
+             "-sip-interface:"
              "l-listen."
              "m-max:"
              "n-no-gatekeeper."
@@ -64,8 +64,10 @@ void CallGen::Main()
              "o-output:"
              "P-prefer:"
              "p-password:"
+             "q-quiet."
              "r-repeat:"
              "-require-gatekeeper."
+             "S-stun:"
              "T-h245tunneldisable."
              "t-trace."
              "-tmaxest:"
@@ -87,37 +89,39 @@ void CallGen::Main()
             "  " << GetFile().GetTitle() << " [options] -l\n"
             "  " << GetFile().GetTitle() << " [options] destination [ destination ... ]\n"
             "where options:\n"
-            "  -l                   Passive/listening mode.\n"
-            "  -m --max num         Maximum number of simultaneous calls\n"
-            "  -r --repeat num      Repeat calls n times\n"
-            "  -C --cycle           Each simultaneous call cycles through destination list\n"
-            "  -t --trace           Trace enable (use multiple times for more detail)\n"
-            "  -o --output file     Specify filename for trace output [stdout]\n"
-            "  --sip-interface addr   Specify IP address and port listen on for SIP [*:5060]\n"
-            "  --h323-interface addr  Specify IP address and port listen on for H.323 [*:1720]\n"
-            "  -g --gatekeeper host Specify gatekeeper host [auto-discover]\n"
-            "  -n --no-gatekeeper   Disable gatekeeper discovery [false]\n"
-            "  --require-gatekeeper Exit if gatekeeper discovery fails [false]\n"
-            "  -u --user username   Specify local username [login name]\n"
-            "  -p --password pwd    Specify gatekeeper H.235 password [none]\n"
-            "  -P --prefer codec    Set codec preference (use multiple times) [none]\n"
-            "  -D --disable codec   Disable codec (use multiple times) [none]\n"
-            "  -f --fast-disable    Disable fast start\n"
-            "  -T --h245tunneldisable  Disable H245 tunnelling.\n"
-            "  -O --out-msg file    Specify PCM16 WAV file for outgoing message [ogm.wav]\n"
-            "  -I --in-dir dir      Specify directory for incoming WAV files [disabled]\n"
-            "  -c --cdr file        Specify Call Detail Record file [none]\n"
-            "  --tcp-base port      Specific the base TCP port to use.\n"
-            "  --tcp-max port       Specific the maximum TCP port to use.\n"
-            "  --udp-base port      Specific the base UDP port to use.\n"
-            "  --udp-max port       Specific the maximum UDP port to use.\n"
-            "  --rtp-base port      Specific the base RTP/RTCP pair of UDP port to use.\n"
-            "  --rtp-max port       Specific the maximum RTP/RTCP pair of UDP port to use.\n"
-            "  --tmaxest  secs      Maximum time to wait for \"Established\" [0]\n"
-            "  --tmincall secs      Minimum call duration in seconds [10]\n"
-            "  --tmaxcall secs      Maximum call duration in seconds [60]\n"
-            "  --tminwait secs      Minimum interval between calls in seconds [10]\n"
-            "  --tmaxwait secs      Maximum interval between calls in seconds [30]\n"
+            "  -l                    Passive/listening mode.\n"
+            "  -q --quiet            Do not display call progress output.\n"
+            "  -m --max num          Maximum number of simultaneous calls\n"
+            "  -r --repeat num       Repeat calls n times\n"
+            "  -C --cycle            Each simultaneous call cycles through destination list\n"
+            "  -t --trace            Trace enable (use multiple times for more detail)\n"
+            "  -o --output file      Specify filename for trace output [stdout]\n"
+            "  -S --stun server      Specify Host/addres of STUN server\n"
+            "  --sip-interface addr  Specify IP address and port listen on for SIP [*:5060]\n"
+            "  --h323-interface addr Specify IP address and port listen on for H.323 [*:1720]\n"
+            "  -g --gatekeeper host  Specify gatekeeper host [auto-discover]\n"
+            "  -n --no-gatekeeper    Disable gatekeeper discovery [false]\n"
+            "  --require-gatekeeper  Exit if gatekeeper discovery fails [false]\n"
+            "  -u --user username    Specify local username [login name]\n"
+            "  -p --password pwd     Specify gatekeeper H.235 password [none]\n"
+            "  -P --prefer codec     Set codec preference (use multiple times) [none]\n"
+            "  -D --disable codec    Disable codec (use multiple times) [none]\n"
+            "  -f --fast-disable     Disable fast start\n"
+            "  -T --h245tunneldisable Disable H245 tunnelling.\n"
+            "  -O --out-msg file     Specify PCM16 WAV file for outgoing message [ogm.wav]\n"
+            "  -I --in-dir dir       Specify directory for incoming WAV files [disabled]\n"
+            "  -c --cdr file         Specify Call Detail Record file [none]\n"
+            "  --tcp-base port       Specific the base TCP port to use.\n"
+            "  --tcp-max port        Specific the maximum TCP port to use.\n"
+            "  --udp-base port       Specific the base UDP port to use.\n"
+            "  --udp-max port        Specific the maximum UDP port to use.\n"
+            "  --rtp-base port       Specific the base RTP/RTCP pair of UDP port to use.\n"
+            "  --rtp-max port        Specific the maximum RTP/RTCP pair of UDP port to use.\n"
+            "  --tmaxest  secs       Maximum time to wait for \"Established\" [0]\n"
+            "  --tmincall secs       Minimum call duration in seconds [10]\n"
+            "  --tmaxcall secs       Maximum call duration in seconds [60]\n"
+            "  --tminwait secs       Minimum interval between calls in seconds [10]\n"
+            "  --tmaxwait secs       Maximum interval between calls in seconds [30]\n"
             "\n"
             "Notes:\n"
             "  If --tmaxest is set a non-zero value then --tmincall is the time to leave\n"
@@ -138,6 +142,8 @@ void CallGen::Main()
   H323EndPoint * h323 = new H323EndPoint(manager);
 #endif
 
+  quietMode = args.HasOption('q');
+
   outgoingMessageFile = args.GetOptionString('O', "ogm.wav");
   if (outgoingMessageFile.IsEmpty())
     cout << "Not using outgoing message file." << endl;
@@ -163,6 +169,9 @@ void CallGen::Main()
     incomingAudioDirectory = PString::Empty();
   }
 
+  if (args.HasOption('S'))
+    manager.SetSTUNServer(args.GetOptionString('S'));
+
 #if OPAL_H323
   {
     PStringArray interfaces = args.GetOptionString("h323-interface").Lines();
@@ -303,24 +312,43 @@ void CallGen::Main()
   if (args.HasOption('g')) {
     PString gkAddr = args.GetOptionString('g');
     cout << "Registering with gatekeeper \"" << gkAddr << "\" ..." << flush;
-    if (h323->UseGatekeeper(gkAddr))
-      cout << "\nGatekeeper set to \"" << *h323->GetGatekeeper() << '"' << endl;
-    else {
+    if (!h323->UseGatekeeper(gkAddr)) {
       cout << "\nError registering with gatekeeper at \"" << gkAddr << '"' << endl;
       return;
     }
   }
   else if (!args.HasOption('n')) {
     cout << "Searching for gatekeeper ..." << flush;
-    if (h323->UseGatekeeper())
-      cout << "\nGatekeeper found: " << *h323->GetGatekeeper() << endl;
-    else {
+    if (!h323->UseGatekeeper()) {
       cout << "\nNo gatekeeper found." << endl;
       if (args.HasOption("require-gatekeeper")) 
         return;
     }
   }
 
+  H323Gatekeeper * gk = h323->GetGatekeeper();
+  if (gk != NULL) {
+    H323Gatekeeper::RegistrationFailReasons reason;
+    while ((reason = gk->GetRegistrationFailReason()) == H323Gatekeeper::UnregisteredLocally)
+      PThread::Sleep(500);
+    switch (reason) {
+      case H323Gatekeeper::RegistrationSuccessful :
+        cout << "\nGatekeeper set to \"" << *gk << '"' << endl;
+        break;
+      case H323Gatekeeper::DuplicateAlias :
+        cout << "\nGatekeeper registration failed: duplicate alias" << endl;
+        break;
+      case H323Gatekeeper::SecurityDenied :
+        cout << "\nGatekeeper registration failed: security denied" << endl;
+        break;
+      case H323Gatekeeper::TransportError :
+        cout << "\nGatekeeper registration failed: transport error" << endl;
+        break;
+      default :
+        cout << "\nGatekeeper registration failed: code=" << reason << endl;
+    }
+  }
+
   if (args.HasOption('f'))
     h323->DisableFastStart(TRUE);
   if (args.HasOption('T'))
@@ -460,21 +488,18 @@ static unsigned RandomRange(PRandom & rand,
 
 
 #define START_OUTPUT(index, token) \
-{ \
-  CallGen::Current().coutMutex.Wait(); \
-  cout << setw(3) << index << ": " << setw(10) << token.Left(10) << ": "
+  if (CallGen::Current().quietMode) ; else { \
+    CallGen::Current().coutMutex.Wait(); \
+    cout << setw(3) << index << ": " << setw(10) << token.Left(10) << ": "
 
 #define END_OUTPUT() \
-  cout << endl; \
-  CallGen::Current().coutMutex.Signal(); \
-}
+    cout << endl; \
+    CallGen::Current().coutMutex.Signal(); \
+  }
 
 static bool generateOutput = false;
 
-#define OUTPUT(index, token, info) \
-  if (generateOutput) { \
-  START_OUTPUT(index, token) << info; END_OUTPUT() \
-  } \
+#define OUTPUT(index, token, info) START_OUTPUT(index, token) << info; END_OUTPUT()
 
 
 void CallThread::Main()
@@ -508,6 +533,7 @@ void CallThread::Main()
       bool stopping = FALSE;
 
       delay = RandomRange(rand, params.tmin_call, params.tmax_call);
+      PTRACE(1, "CallGen\tMax call time " << delay);
 
       START_OUTPUT(index, token) << "Making call " << count;
       if (params.repeat)
@@ -519,11 +545,13 @@ void CallThread::Main()
 
       if (params.tmax_est > 0) {
         OUTPUT(index, token, "Waiting " << params.tmax_est << " seconds for establishment");
+        PTRACE(1, "CallGen\tWaiting " << params.tmax_est << " seconds for establishment");
 
         PTimer timeout = params.tmax_est;
         while (!callgen.manager.IsCallEstablished(token)) {
           stopping = exit.Wait(100);
           if (stopping || !timeout.IsRunning() || !callgen.manager.HasCall(token)) {
+            PTRACE(1, "CallGen\tTimeout/Stopped on establishment");
             delay = 0;
             break;
           }
@@ -538,6 +566,7 @@ void CallThread::Main()
 
       // end the call
       OUTPUT(index, token, "Clearing call");
+      PTRACE(1, "CallGen\tClearing call");
 
       callgen.manager.ClearCallSynchronous(token);
 
@@ -566,8 +595,9 @@ void CallThread::Main()
 
 void CallThread::Stop()
 {
-  if (!IsTerminated())
+  if (!IsTerminated()) {
     OUTPUT(index, PString::Empty(), "Stopping.");
+  }
 
   exit.Signal();
 }
