@@ -21,9 +21,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21158 $
+ * $Revision: 22141 $
  * $Author: rjongbloed $
- * $Date: 2008-09-25 03:03:10 +0000 (Thu, 25 Sep 2008) $
+ * $Date: 2009-03-02 20:26:18 -0600 (Mon, 02 Mar 2009) $
  */
 
 #include <ptlib.h>
@@ -47,8 +47,6 @@
 #include <t38/t38proto.h>
 #endif
 
-#include <t38/t38proto.h>
-
 #include <opal/transcoders.h>
 #include <lids/lidep.h>
 #include <ptclib/pstun.h>
@@ -171,9 +169,9 @@ void SimpleOpalProcess::Main()
 
 
   if (args.HasOption('h') || (!args.HasOption('l') && args.GetCount() == 0)) {
-    cout << "Usage : " << GetName() << " [options] -l\n"
-            "      : " << GetName() << " [options] [alias@]hostname   (no gatekeeper)\n"
-            "      : " << GetName() << " [options] alias[@hostname]   (with gatekeeper)\n"
+    cout << "Usage : " << GetFile().GetTitle() << " [options] -l\n"
+            "      : " << GetFile().GetTitle() << " [options] [alias@]hostname   (no gatekeeper)\n"
+            "      : " << GetFile().GetTitle() << " [options] alias[@hostname]   (with gatekeeper)\n"
             "General options:\n"
             "  -l --listen             : Listen for incoming calls.\n"
             "  -d --dial-peer spec     : Set dial peer for routing calls (see below)\n"
@@ -332,9 +330,9 @@ void SimpleOpalProcess::Main()
 "      iax2:.* = pots:<dn>\n"
 "\n"
 "    If POTS is not enabled and the PC sound system is enabled:\n"
-"      iax2:.* = pc:<du>\n"
-"      h323:.* = pc:<du>\n"
-"      sip:. * = pc:<du>\n"
+"      iax2:.* = pc:\n"
+"      h323:.* = pc:\n"
+"      sip:. * = pc:\n"
 "\n"
 #if OPAL_IVR
 "    If IVR is enabled then a # from any protocol will route it it, ie:\n"
@@ -343,7 +341,8 @@ void SimpleOpalProcess::Main()
 #endif
 #if OPAL_IAX2
 "    If IAX2 is enabled then you can make a iax2 call with a command like:\n"
-"       simpleopal -IHn  iax2:guest@misery.digium.com/s\n"
+"       simpleopal -I -H  iax2:guest@misery.digium.com/s\n"
+"           ((Please ensure simplopal is the only iax2 app running on your box))\n"
 #endif
             << endl;
     return;
@@ -390,7 +389,6 @@ MyManager::MyManager()
 #endif
 #if OPAL_FAX
   faxEP = NULL;
-  t38EP = NULL;
 #endif
 
   pauseBeforeDialing = PFalse;
@@ -412,13 +410,13 @@ PBoolean MyManager::Initialise(PArgList & args)
 #if OPAL_VIDEO
   // Set the various global options
   if (args.HasOption("rx-video"))
-    autoStartReceiveVideo = PTrue;
+    SetAutoStartReceiveVideo(true);
   if (args.HasOption("no-rx-video"))
-    autoStartReceiveVideo = PFalse;
+    SetAutoStartReceiveVideo(false);
   if (args.HasOption("tx-video"))
-    autoStartTransmitVideo = PTrue;
+    SetAutoStartTransmitVideo(true);
   if (args.HasOption("no-tx-video"))
-    autoStartTransmitVideo = PFalse;
+    SetAutoStartTransmitVideo(false);
 
   if (args.HasOption("grabber")) {
     PVideoDevice::OpenArgs video = GetVideoInputDevice();
@@ -608,6 +606,12 @@ PBoolean MyManager::Initialise(PArgList & args)
   if (!args.HasOption("no-iax2")) {
     iax2EP = new IAX2EndPoint(*this);
     
+    if (!iax2EP->InitialisedOK()) {
+      cerr << "IAX2 Endpoint is not initialised correctly" << endl;
+      cerr << "Is there another application using port 4569? " << endl;
+      return PFalse;
+    }
+
     if (args.HasOption('p'))
       iax2EP->SetPassword(args.GetOptionString('p'));
     
@@ -700,9 +704,6 @@ PBoolean MyManager::Initialise(PArgList & args)
   {
     OpalMediaFormat fmt(OpalT38); // Force instantiation of T.38 media format
     faxEP = new OpalFaxEndPoint(*this);
-    t38EP = new OpalT38EndPoint(*this);
-
-    allMediaFormats += t38EP->GetMediaFormats();
     allMediaFormats += faxEP->GetMediaFormats();
   }
 #endif
@@ -770,28 +771,29 @@ PBoolean MyManager::Initialise(PArgList & args)
 #endif // OPAL_LID
     if (pcssEP != NULL) {
 #if OPAL_H323
-      AddRouteEntry("h323:.* = pc:<du>");
+      AddRouteEntry("h323:.* = pc:");
 #if OPAL_PTLIB_SSL
       //if (h323sEP != NULL) 
-        AddRouteEntry("h323s:.* = pc:<du>");
+        AddRouteEntry("h323s:.* = pc:");
 #endif
 #endif
 #if OPAL_SIP
-      AddRouteEntry("sip:.*  = pc:<du>");
+      AddRouteEntry("sip:.*  = pc:");
 #endif
     }
   }
                                                                                                                                             
 #if OPAL_IAX2
   if (pcssEP != NULL) {
-    AddRouteEntry("iax2:.*  = pc:<du>");
+    AddRouteEntry("iax2:.*  = pc:");
     AddRouteEntry("pc:.*   = iax2:<da>");
   }
 #endif
 
 #if OPAL_FAX
-  if (t38EP != NULL) {
-      AddRouteEntry("sip:.*  = t38:<da>");
+  if (faxEP != NULL) {
+    AddRouteEntry("sip:.*  = t38:<da>");
+    AddRouteEntry("sip:.*  = fax:<da>");
   }
 #endif
 
@@ -847,6 +849,7 @@ PBoolean MyManager::Initialise(PArgList & args)
         unsigned rate = args.GetOptionString("video-rate").AsUnsigned();
         unsigned frameTime = 90000 / rate;
         mediaFormat.SetOptionInteger(OpalMediaFormat::FrameTimeOption(), frameTime);
+        mediaFormat.SetOptionBoolean("Rate Control Enable", true);
       }
       OpalMediaFormat::SetRegisteredMediaFormat(mediaFormat);
     }
