@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21785 $
+ * $Revision: 22197 $
  * $Author: rjongbloed $
- * $Date: 2008-12-12 03:07:57 +0000 (Fri, 12 Dec 2008) $
+ * $Date: 2009-03-11 20:11:14 -0500 (Wed, 11 Mar 2009) $
  */
 
 #include <ptlib.h>
@@ -689,6 +689,8 @@ unsigned H323_RTPChannel::GetSessionID() const
 
 PBoolean H323_RTPChannel::OnSendingPDU(H245_H2250LogicalChannelParameters & param) const
 {
+  param.m_sessionID = connection.GetExternalSessionID(GetSessionID(), GetCapability().GetMediaFormat().GetMediaType());
+
   return rtpCallbacks.OnSendingPDU(*this, param);
 }
 
@@ -1097,7 +1099,7 @@ void H323DataChannel::OnSendOpenAck(const H245_OpenLogicalChannel & open,
   H323TransportAddress address;
   param->IncludeOptionalField(H245_H2250LogicalChannelAckParameters::e_mediaChannel);
   if (listener != NULL)
-    address = listener->GetLocalAddress(connection.GetControlChannel().GetLocalAddress());
+    address = listener->GetLocalAddress(connection.GetControlChannel().GetRemoteAddress());
   else
     address = transport->GetLocalAddress();
 
