@@ -33,9 +33,9 @@
  *
  * Contributor(s): Many thanks to Simon Horne.
  *
- * $Revision: 21004 $
+ * $Revision: 22680 $
  * $Author: rjongbloed $
- * $Date: 2008-09-16 07:08:56 +0000 (Tue, 16 Sep 2008) $
+ * $Date: 2009-05-20 19:48:57 -0500 (Wed, 20 May 2009) $
  */
 
 #include <ptlib.h>
@@ -1006,17 +1006,17 @@ H460_FeatureSet::H460_FeatureSet(const H225_FeatureSet & fs)
 	CreateFeatureSet(fs);
 }
 
-H460_FeatureSet::H460_FeatureSet(const H225_ArrayOf_GenericData & generic)
+H460_FeatureSet::H460_FeatureSet(const H225_ArrayOf_GenericData & genericData)
 {
-	Features.DisallowDeleteObjects();   // Built FeatureSet should not delete Objects.
-	ep = NULL;
-	baseSet = NULL;
+  Features.DisallowDeleteObjects();   // Built FeatureSet should not delete Objects.
+  ep = NULL;
+  baseSet = NULL;
 
-    for (PINDEX i=0; i < generic.GetSize(); i++) {
-	   AddFeature((H460_Feature *)&generic[i]);
-	}
+  for (PINDEX i=0; i < genericData.GetSize(); i++)
+    AddFeature((H460_Feature *)&genericData[i]);
 }
 
+
 PBoolean H460_FeatureSet::ProcessFirstPDU(const H225_FeatureSet & fs)
 {
   
