@@ -24,14 +24,16 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21145 $
- * $Author: rjongbloed $
- * $Date: 2008-09-24 00:15:51 +0000 (Wed, 24 Sep 2008) $
+ * $Revision: 21447 $
+ * $Author: csoutheren $
+ * $Date: 2008-11-04 01:37:55 -0600 (Tue, 04 Nov 2008) $
  */
 
 #include <opal/buildopts.h>
 
+#pragma warning(disable:4206)
+
 #if OPAL_JAVA
-#include "java_swig_wrapper.c"
+#include "java_swig_wrapper.inc"
 #endif
 
