@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22197 $
+ * $Revision: 23601 $
  * $Author: rjongbloed $
- * $Date: 2009-03-12 01:11:14 +0000 (Thu, 12 Mar 2009) $
+ * $Date: 2009-10-07 02:53:21 -0500 (Wed, 07 Oct 2009) $
  */
 
 #include <ptlib.h>
@@ -254,7 +254,7 @@ static void AddTransportAddress(OpalTransportAddressArray & interfaceAddresses,
                                 WORD port,
                                 const PString & proto)
 {
-  if (ip == natInterfaceIP && ip != natExternalIP)
+  if (ip != natExternalIP && (natInterfaceIP.IsAny() || ip == natInterfaceIP))
     AddTransportAddress(interfaceAddresses, natInterfaceIP, natExternalIP, natExternalIP, port, proto);
 
   OpalTransportAddress addr(ip, port, proto);
@@ -309,7 +309,7 @@ OpalTransportAddressArray OpalEndPoint::GetInterfaceAddresses(PBoolean excludeLo
   OpalTransportAddressArray interfaceAddresses;
 
   OpalTransportAddress associatedLocalAddress, associatedRemoteAddress;
-  PIPSocket::Address natInterfaceIP = PIPSocket::GetDefaultIpAny();
+  PIPSocket::Address natInterfaceIP;
   PIPSocket::Address natExternalIP;
   if (associatedTransport != NULL) {
     associatedLocalAddress = associatedTransport->GetLocalAddress();
@@ -323,6 +323,10 @@ OpalTransportAddressArray OpalEndPoint::GetInterfaceAddresses(PBoolean excludeLo
       natMethod->GetInterfaceAddress(natInterfaceIP);
       natMethod->GetExternalAddress(natExternalIP);
     }
+    else if (manager.GetTranslationAddress().IsValid()) {
+      natInterfaceIP = PIPSocket::GetDefaultIpAny();
+      natExternalIP = manager.GetTranslationAddress();
+    }
   }
 
   OpalListenerList::iterator listener;
