@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 21999 $
+ * $Revision: 22630 $
  * $Author: rjongbloed $
- * $Date: 2009-02-05 08:11:20 +0000 (Thu, 05 Feb 2009) $
+ * $Date: 2009-05-18 01:12:59 -0500 (Mon, 18 May 2009) $
  */
 
 #include <ptlib.h>
@@ -207,9 +207,14 @@ void OpalIVRConnection::OnEstablished()
 
 PBoolean OpalIVRConnection::StartVXML()
 {
+  if (vxmlSession.IsPlaying()) {
+    PTRACE(4, "IVR\tStartVXML, already playing");
+    return true;
+  }
+
   PStringToString & vars = vxmlSession.GetSessionVars();
 
-  PString originator = m_connStringOptions("Originator-Address");
+  PString originator = m_connStringOptions(OPAL_OPT_ORIGINATOR_ADDRESS);
   if (originator.IsEmpty())
     originator = m_connStringOptions("Remote-Address");
   if (!originator.IsEmpty()) {
@@ -225,8 +230,14 @@ PBoolean OpalIVRConnection::StartVXML()
   if (vxmlToLoad.IsEmpty())
     return endpoint.StartVXML();
 
-  if (vxmlToLoad.Find("<?xml") == 0)
+  if (vxmlToLoad.Find("<?xml") == 0) {
+    PTRACE(4, "IVR\tStartVXML:\n" << vxmlToLoad);
     return vxmlSession.LoadVXML(vxmlToLoad);
+  }
+
+  PFilePath vxmlFile = PURL(vxmlToLoad).AsFilePath();
+  if (vxmlFile.GetType() *= ".vxml")
+    return vxmlSession.LoadFile(vxmlFile);
 
   ////////////////////////////////
 
@@ -234,29 +245,30 @@ PBoolean OpalIVRConnection::StartVXML()
   PINDEX delay = 0;
   PString voice;
 
-  // always start with 500ms of silence
-  vxmlSession.PlaySilence(500);
-
   PINDEX i;
   PStringArray tokens = vxmlToLoad.Tokenise(';', PFalse);
   for (i = 0; i < tokens.GetSize(); ++i) {
     PString str(tokens[i]);
 
-    if (str.Find("file://") == 0) {
-      PString fn = str.Mid(7).Trim();
+    if (str.Find("file:") == 0) {
+      PURL url = str;
+      if (url.IsEmpty()) {
+        PTRACE(2, "IVR\tInvalid URL \"" << str << '"');
+        continue;
+      }
+
+      PFilePath fn = url.AsFilePath();
+      if (fn.IsEmpty()) {
+        PTRACE(2, "IVR\tUnsupported host in URL " << url);
+        continue;
+      }
+
+      if (!voice.IsEmpty())
+        fn = fn.GetDirectory() + voice + PDIR_SEPARATOR + fn.GetFileName();
+
       PTRACE(3, "IVR\tPlaying file " << fn);
-      if (fn.Right(5) *= ".vxml") {
-        if (!voice.IsEmpty())
-          fn = voice + PDIR_SEPARATOR + fn;
-        vxmlSession.LoadFile(fn);
-        continue;
-      }
-      else if (fn.Right(4) *= ".wav") {
-        if (!voice.IsEmpty())
-          fn = voice + PDIR_SEPARATOR + fn;
-        vxmlSession.PlayFile(fn, repeat, delay);
-        continue;
-      }
+      vxmlSession.PlayFile(fn, repeat, delay);
+      continue;
     }
 
     PINDEX pos = str.Find("=");
@@ -308,6 +320,7 @@ PBoolean OpalIVRConnection::StartVXML()
   }
 
   vxmlSession.PlayStop();
+  vxmlSession.Trigger();
 
   return PTrue;
 }
@@ -437,12 +450,6 @@ PBoolean OpalIVRMediaStream::Open()
   return PFalse;
 }
 
-PBoolean OpalIVRMediaStream::ReadPacket(RTP_DataFrame & packet)
-{
-  return OpalRawMediaStream::ReadPacket(packet);
-}
-
-
 PBoolean OpalIVRMediaStream::IsSynchronous() const
 {
   return true;
