@@ -24,9 +24,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22166 $
+ * $Revision: 23874 $
  * $Author: rjongbloed $
- * $Date: 2009-03-06 00:29:00 +0000 (Fri, 06 Mar 2009) $
+ * $Date: 2009-12-13 21:45:30 -0600 (Sun, 13 Dec 2009) $
  */
 
 #include <ptlib.h>
@@ -274,14 +274,14 @@ void OpalManager::AttachEndPoint(OpalEndPoint * endpoint, const PString & prefix
   PWriteWaitAndSignal mutex(endpointsMutex);
 
   if (endpointMap.find(thePrefix) != endpointMap.end()) {
-    PTRACE(1, "OpalMan\tCannot re-register endpoint prefix " << thePrefix);
+    PTRACE(1, "OpalMan\tCannot re-attach endpoint prefix " << thePrefix);
     return;
   }
 
   if (endpointList.GetObjectsIndex(endpoint) == P_MAX_INDEX)
     endpointList.Append(endpoint);
   endpointMap[thePrefix] = endpoint;
-  PTRACE(1, "OpalMan\tRegistered endpoint with prefix " << thePrefix);
+  PTRACE(3, "OpalMan\tAttached endpoint with prefix " << thePrefix);
 }
 
 
@@ -712,7 +712,7 @@ PBoolean OpalManager::IsMediaBypassPossible(const OpalConnection & source,
 }
 
 
-PBoolean OpalManager::OnOpenMediaStream(OpalConnection & connection,
+PBoolean OpalManager::OnOpenMediaStream(OpalConnection & PTRACE_PARAM(connection),
                                         OpalMediaStream & PTRACE_PARAM(stream))
 {
   PTRACE(3, "OpalMan\tOnOpenMediaStream " << connection << ',' << stream);
@@ -1653,7 +1653,7 @@ bool OpalWAVRecordManager::Mixer_T::Open(const PFilePath & fn, bool mono)
 
   m_file.SetFormat(OpalWAVFile::fmt_PCM);
   if (!m_file.Open(fn, PFile::ReadWrite, PFile::Create|PFile::Truncate)) {
-    PTRACE(2, "OPAL\tCould not open file \"" << fn << '"');
+    PTRACE(2, "OPAL\tCould not open file \"" << fn << "\": " << m_file.GetErrorText());
     return false;
   }
 
