@@ -26,9 +26,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22553 $
- * $Author: csoutheren $
- * $Date: 2009-05-07 08:56:06 +0000 (Thu, 07 May 2009) $
+ * $Revision: 24346 $
+ * $Author: rjongbloed $
+ * $Date: 2010-05-17 20:29:28 -0500 (Mon, 17 May 2010) $
  */
 
 #include <ptlib.h>
@@ -1515,7 +1515,10 @@ void OpalManager_C::HandleTransferCall(const OpalMessage & command, OpalMessageB
     }
   }
 
-  connection->TransferConnection(command.m_param.m_callSetUp.m_partyB);
+  if (connection->GetPhase() < OpalConnection::ConnectedPhase)
+    connection->ForwardCall(command.m_param.m_callSetUp.m_partyB);
+  else
+    connection->TransferConnection(command.m_param.m_callSetUp.m_partyB);
 }
 
 
