@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 22616 $
+ * $Revision: 23991 $
  * $Author: rjongbloed $
- * $Date: 2009-05-14 02:02:29 +0000 (Thu, 14 May 2009) $
+ * $Date: 2010-01-29 02:44:30 -0600 (Fri, 29 Jan 2010) $
  */
 
 #include <ptlib.h>
@@ -46,8 +46,6 @@
 #include <opal/call.h>
 #include <sip/handlers.h>
 
-#define SIP_THREAD_POOL   1
-
 
 #define new PNEW
 
@@ -78,6 +76,7 @@ SIPEndPoint::SIPEndPoint(OpalManager & mgr)
 #if OPAL_HAS_SIPIM
   , m_sipIMManager(*this)
 #endif
+  , m_disableTrying(true)
 
 #ifdef _MSC_VER
 #pragma warning(default:4355)
@@ -125,7 +124,7 @@ void SIPEndPoint::ShutDown()
   bool shuttingDown = true;
   while (shuttingDown) {
     shuttingDown = false;
-    PSafePtr<SIPHandler> handler(activeSIPHandlers, PSafeReference);
+    PSafePtr<SIPHandler> handler = activeSIPHandlers.GetFirstHandler();
     while (handler != NULL) {
       if (handler->ShutDown())
         activeSIPHandlers.Remove(handler++);
@@ -160,29 +159,62 @@ PString SIPEndPoint::GetDefaultTransport() const
 
 PBoolean SIPEndPoint::NewIncomingConnection(OpalTransport * transport)
 {
-  PTRACE_IF(2, transport->IsReliable(), "SIP\tListening thread started.");
-
   transport->SetBufferSize(SIP_PDU::MaxSize);
 
-  do {
-    HandlePDU(*transport);
-  } while (transport->IsOpen() && transport->IsReliable() && !transport->bad() && !transport->eof());
+  if (transport->IsReliable()) {
+    // TCP connection, read until closed
+    TransportReadLoop(transport);
+    return false; // Do not delete the transport!
+  }
 
-  PTRACE_IF(2, transport->IsReliable(), "SIP\tListening thread finished.");
-
-  return PTrue;
+  // UDP packet, handle it and return
+  HandlePDU(*transport);
+  return true;
 }
 
 
 void SIPEndPoint::TransportThreadMain(PThread &, INT param)
+{
+  TransportReadLoop((OpalTransport *)param);
+}
+
+
+void SIPEndPoint::TransportReadLoop(OpalTransport * transport)
 {
   PTRACE(4, "SIP\tRead thread started.");
-  OpalTransport * transport = (OpalTransport *)param;
 
   do {
     HandlePDU(*transport);
   } while (transport->IsOpen() && !transport->bad() && !transport->eof());
-  
+
+  /* This is increadibly ugly, but a true fix requires quite a substantial rewrite.
+
+     The problem is that a TCP connection is SIP could be associated with multiple
+     SIP connection objects, that is INVITEs to different calls could come down
+     the same TCP connection. At present there is a pointer to the TCP transport
+     in each SIPConnection instance and we have to wait for all of this connections
+     to finish with the pointer before we delete the transport.
+
+     This clearly requires a reference counting system, or better yet removal of the
+     pointer to the transport completely so all writes of PDU's to a remote are done
+     via a function on the endpoint which automatically selects the appropriate
+     transport from a poll it has.
+   */
+
+  PTRACE(4, "SIP\tRead transport closed, checking connections that use it.");
+  PSafePtr<SIPConnection> connection = PSafePtrCast<OpalConnection, SIPConnection>(connectionsActive.GetAt(0, PSafeReference));
+  while (connection != NULL) {
+    if (&connection->GetTransport() == transport) {
+      connection->Release(OpalConnection::EndedByTransportFail);
+      while (&connection->GetTransport() == transport)
+        PThread::Sleep(100);
+    }
+
+    ++connection;
+  }
+
+  delete connection;
+
   PTRACE(4, "SIP\tRead thread finished.");
 }
 
@@ -194,7 +226,7 @@ void SIPEndPoint::NATBindingRefresh(PTimer &, INT)
 
   if (natMethod != None) {
     PTRACE(5, "SIP\tNAT Binding refresh started.");
-    for (PSafePtr<SIPHandler> handler(activeSIPHandlers, PSafeReadOnly); handler != NULL; ++handler) {
+    for (PSafePtr<SIPHandler> handler = activeSIPHandlers.GetFirstHandler(PSafeReadOnly); handler != NULL; ++handler) {
 
       OpalTransport * transport = NULL;
       if (handler->GetState () != SIPHandler::Subscribed ||
@@ -254,12 +286,9 @@ OpalTransport * SIPEndPoint::CreateTransport(const SIPURL & remoteURL, const PSt
   }
 
   if (transport == NULL) {
-    // No compatible listeners, can't use their binding
-    transport = remoteAddress.CreateTransport(*this, OpalTransportAddress::NoBinding);
-    if (transport == NULL) {
-      PTRACE(1, "SIP\tCould not create transport for " << remoteAddress);
+    // No compatible listeners, can't create a transport to send if we cannot hear the responses!
+    PTRACE(2, "SIP\tNo compatible listener to create transport for " << remoteAddress);
       return NULL;
-    }
   }
 
   if (!transport->SetRemoteAddress(remoteAddress)) {
@@ -320,14 +349,9 @@ static PString TranslateENUM(const PString & remoteParty)
   if (remoteParty.Find('@') == P_MAX_INDEX) {
 
     // make sure the number has only digits
-    PString e164 = remoteParty;
-    PINDEX pos = e164.Find(':');
-    if (pos != P_MAX_INDEX)
-      e164.Delete(0, pos+1);
-    pos = e164.FindSpan("0123456789*#", e164[0] != '+' ? 0 : 1);
-    e164.Delete(pos, P_MAX_INDEX);
-
-    if (!e164.IsEmpty()) {
+    PINDEX pos = remoteParty.Find(':');
+    PString e164 = pos != P_MAX_INDEX ? remoteParty.Mid(pos+1) : remoteParty;
+    if (e164.FindSpan("0123456789*#", e164[0] != '+' ? 0 : 1) == P_MAX_INDEX) {
       PString str;
       if (PDNS::ENUMLookup(e164, "E2U+SIP", str)) {
         PTRACE(4, "SIP\tENUM converted remote party " << remoteParty << " to " << str);
@@ -359,7 +383,8 @@ PBoolean SIPEndPoint::MakeConnection(OpalCall & call,
 
 void SIPEndPoint::OnReleased(OpalConnection & connection)
 {
-  m_receivedConnectionTokens.RemoveAt(connection.GetIdentifier());
+  if (m_receivedConnectionTokens(connection.GetIdentifier()) == connection.GetToken())
+    m_receivedConnectionTokens.RemoveAt(connection.GetIdentifier());
   OpalEndPoint::OnReleased(connection);
 }
 
@@ -382,7 +407,7 @@ PBoolean SIPEndPoint::GarbageCollection()
   bool transactionsDone = transactions.DeleteObjectsToBeRemoved();
 
 
-  PSafePtr<SIPHandler> handler(activeSIPHandlers, PSafeReference);
+  PSafePtr<SIPHandler> handler = activeSIPHandlers.GetFirstHandler();
   while (handler != NULL) {
     // If unsubscribed then we do the shut down to clean up the handler
     if (handler->GetState() == SIPHandler::Unsubscribed && handler->ShutDown())
@@ -492,7 +517,7 @@ PBoolean SIPEndPoint::OnReceivedPDU(OpalTransport & transport, SIP_PDU * pdu)
     case SIP_PDU::Method_CANCEL :
       token = m_receivedConnectionTokens(mime.GetCallID());
       if (!token.IsEmpty()) {
-        AddWork(new SIP_PDU_Work(*this, token, pdu));
+        m_connectionThreadPool.AddWork(new SIP_PDU_Work(*this, token, pdu), token);
         return true;
       }
       break;
@@ -501,8 +526,26 @@ PBoolean SIPEndPoint::OnReceivedPDU(OpalTransport & transport, SIP_PDU * pdu)
       if (toToken.IsEmpty()) {
         token = m_receivedConnectionTokens(mime.GetCallID());
         if (!token.IsEmpty()) {
-          AddWork(new SIP_PDU_Work(*this, token, pdu));
-          return true;
+          PSafePtr<SIPConnection> connection = GetSIPConnectionWithLock(token, PSafeReference);
+          if (connection != NULL) {
+            switch (connection->CheckINVITE(*pdu)) {
+              case SIPConnection::IsNewINVITE : // Process new INVITE
+                break;
+
+              case SIPConnection::IsDuplicateINVITE : // Completely ignore duplicate INVITE
+                return true;
+
+              case SIPConnection::IsReINVITE : // Pass on to worker thread if re-INVITE
+                m_connectionThreadPool.AddWork(new SIP_PDU_Work(*this, token, pdu), token);
+                return true;
+
+              case SIPConnection::IsLoopedINVITE : // Send back error if looped INVITE
+                SIP_PDU response(*pdu, SIP_PDU::Failure_LoopDetected);
+                response.GetMIME().SetProductInfo(GetUserAgent(), connection->GetProductInfo());
+                pdu->SendResponse(transport, response);
+                return true;
+            }
+          }
         }
 
         pdu->SendResponse(transport, SIP_PDU::Information_Trying, this);
@@ -517,7 +560,8 @@ PBoolean SIPEndPoint::OnReceivedPDU(OpalTransport & transport, SIP_PDU * pdu)
       // Do next case
 
     default :
-      pdu->SendResponse(transport, SIP_PDU::Information_Trying, this);
+      if (!m_disableTrying || (pdu->GetMethod() == SIP_PDU::Method_INVITE))
+        pdu->SendResponse(transport, SIP_PDU::Information_Trying, this);
       // Do next case
 
     case SIP_PDU::Method_ACK :
@@ -525,8 +569,7 @@ PBoolean SIPEndPoint::OnReceivedPDU(OpalTransport & transport, SIP_PDU * pdu)
       break;
 
     case SIP_PDU::NumMethods :
-      AddWork(new SIP_PDU_Work(*this, token, pdu));
-      return true;
+      break;
   }
 
   if (hasToConnection)
@@ -536,28 +579,22 @@ PBoolean SIPEndPoint::OnReceivedPDU(OpalTransport & transport, SIP_PDU * pdu)
   else
     return OnReceivedConnectionlessPDU(transport, pdu);
 
-  AddWork(new SIP_PDU_Work(*this, token, pdu));
+  m_connectionThreadPool.AddWork(new SIP_PDU_Work(*this, token, pdu), token);
   return true;
 }
 
-void SIPEndPoint::AddWork(SIP_PDU_Work * work)
-{
-#if SIP_THREAD_POOL
-  threadPool.AddWork(work, work->m_token);
-#else
-  PTRACE(2, "SIP\tStarted processing PDU");
-  work->OnReceivedPDU();
-  PTRACE(2, "SIP\tFinished processing PDU");
-  delete work;
-#endif
-}
 
 bool SIPEndPoint::OnReceivedConnectionlessPDU(OpalTransport & transport, SIP_PDU * pdu)
 {
   if (pdu->GetMethod() == SIP_PDU::NumMethods || pdu->GetMethod() == SIP_PDU::Method_CANCEL) {
-    PSafePtr<SIPTransaction> transaction = GetTransaction(pdu->GetTransactionID(), PSafeReference);
-    if (transaction != NULL)
-      transaction->OnReceivedResponse(*pdu);
+    PString id;
+    if (activeSIPHandlers.FindSIPHandlerByCallID(id = pdu->GetMIME().GetCallID(), PSafeReference) != NULL ||
+        GetTransaction(id = pdu->GetTransactionID(), PSafeReference) != NULL) {
+      m_handlerThreadPool.AddWork(new SIP_PDU_Work(*this, id, pdu), id);
+      return true;
+    }
+
+    PTRACE(2, "SIP\tReceived response for unmatched transaction, id=" << id);
     return false;
   }
 
@@ -638,6 +675,7 @@ PBoolean SIPEndPoint::OnReceivedSUBSCRIBE(OpalTransport & transport, SIP_PDU & p
     dialog.SetRouteSet(mime.GetRecordRoute(true));
 
     handler = new SIPNotifyHandler(*this, dialog.GetRemoteURI().AsString(), eventPackage, dialog);
+    handler.SetSafetyMode(PSafeReadWrite);
     activeSIPHandlers.Append(handler);
 
     handler->GetTransport()->SetInterface(transport.GetInterface());
@@ -662,7 +700,7 @@ PBoolean SIPEndPoint::OnReceivedSUBSCRIBE(OpalTransport & transport, SIP_PDU & p
   if (expires > 0)
     handler->SendNotify(NULL);
   else
-    handler->SendRequest(SIPHandler::Unsubscribing);
+    handler->ActivateState(SIPHandler::Unsubscribing);
 
   return true;
 }
@@ -670,49 +708,11 @@ PBoolean SIPEndPoint::OnReceivedSUBSCRIBE(OpalTransport & transport, SIP_PDU & p
 
 void SIPEndPoint::OnReceivedResponse(SIPTransaction & transaction, SIP_PDU & response)
 {
-  PSafePtr<SIPHandler> handler = NULL;
- 
-  if (transaction.GetMethod() == SIP_PDU::Method_REGISTER
-      || transaction.GetMethod() == SIP_PDU::Method_SUBSCRIBE
-      || transaction.GetMethod() == SIP_PDU::Method_PUBLISH
-      || transaction.GetMethod() == SIP_PDU::Method_MESSAGE) {
-    // Have a response to various non-INVITE messages
-    handler = activeSIPHandlers.FindSIPHandlerByCallID(transaction.GetMIME().GetCallID(), PSafeReadWrite);
-    if (handler == NULL) 
-      return;
-  }
-
-  switch (response.GetStatusCode()) {
-    case SIP_PDU::Failure_IntervalTooBrief :
-      OnReceivedIntervalTooBrief(transaction, response);
-      break;
-
-    case SIP_PDU::Failure_UnAuthorised :
-    case SIP_PDU::Failure_ProxyAuthenticationRequired :
-      OnReceivedAuthenticationRequired(transaction, response);
-      break;
-
-    case SIP_PDU::Failure_RequestTimeout :
-      if (handler != NULL)
-        handler->OnTransactionFailed(transaction);
-      break;
-
-    default :
-      switch (response.GetStatusCode()/100) {
-        case 1 :
-          // Do nothing on 1xx
-          break;
-
-        case 2 :
-          OnReceivedOK(transaction, response);
-          break;
-
-        default :
-          // Failure for a SUBSCRIBE/REGISTER/PUBLISH/MESSAGE 
-          if (handler != NULL)
-            handler->OnFailed(response.GetStatusCode());
-          break;
-      }
+  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByCallID(response.GetMIME().GetCallID(), PSafeReadWrite);
+  if (handler != NULL)
+    handler->OnReceivedResponse(transaction, response);
+  else {
+    PTRACE(2, "SIP\tResponse received for unknown handler ID: " << response.GetMIME().GetCallID());
   }
 }
 
@@ -778,9 +778,22 @@ PBoolean SIPEndPoint::OnReceivedINVITE(OpalTransport & transport, SIP_PDU * requ
   // parse the incoming To field, and check if we accept incoming calls for this address
   SIPURL toAddr(mime.GetTo());
   if (!IsAcceptedAddress(toAddr)) {
-    PTRACE(2, "SIP\tIncoming INVITE from " << request->GetURI() << " for unacceptable address " << toAddr);
+    PTRACE(2, "SIP\tIncoming INVITE for " << request->GetURI() << " for unacceptable address " << toAddr);
     request->SendResponse(transport, SIP_PDU::Failure_NotFound, this);
-    return PFalse;
+    return false;
+  }
+
+  if (!request->GetEntityBody().IsEmpty() &&
+         (!mime.GetContentEncoding().IsEmpty() ||
+           mime.GetContentType() != "application/sdp")) {
+    // Do not currently support anything other than SDP, in particular multipart stuff.
+    PTRACE(2, "SIP\tIncoming INVITE for " << request->GetURI() << " does not contain SDP");
+    SIP_PDU response(*request, SIP_PDU::Failure_UnsupportedMediaType);
+    response.GetMIME().SetAccept("application/sdp");
+    response.GetMIME().SetAcceptEncoding("identity");
+    response.SetAllow(GetAllowedMethods());
+    request->SendResponse(transport, response, this);
+    return false;
   }
 
   // See if we are replacing an existing call.
@@ -790,7 +803,7 @@ PBoolean SIPEndPoint::OnReceivedINVITE(OpalTransport & transport, SIP_PDU * requ
     PSafePtr<SIPConnection> replacedConnection = GetSIPConnectionWithLock(mime("Replaces"), PSafeReference, &errorCode);
     if (replacedConnection == NULL) {
       PTRACE_IF(2, errorCode==SIP_PDU::Failure_BadRequest,
-                "SIP\tBad Replaces header in INVITE from " << request->GetURI());
+                "SIP\tBad Replaces header in INVITE for " << request->GetURI());
       PTRACE_IF(2, errorCode==SIP_PDU::Failure_TransactionDoesNotExist,
                 "SIP\tNo connection matching dialog info in Replaces header of INVITE from " << request->GetURI());
       request->SendResponse(transport, errorCode, this);
@@ -809,7 +822,7 @@ PBoolean SIPEndPoint::OnReceivedINVITE(OpalTransport & transport, SIP_PDU * requ
   else {
     newTransport = CreateTransport(SIPURL("", transport.GetRemoteAddress(), 0), transport.GetInterface());
     if (newTransport == NULL) {
-      PTRACE(1, "SIP\tFailed to create transport for SIPConnection for INVITE from " << request->GetURI() << " for " << toAddr);
+      PTRACE(1, "SIP\tFailed to create transport for SIPConnection for INVITE for " << request->GetURI() << " to " << toAddr);
       request->SendResponse(transport, SIP_PDU::Failure_NotFound, this);
       return PFalse;
     }
@@ -832,68 +845,29 @@ PBoolean SIPEndPoint::OnReceivedINVITE(OpalTransport & transport, SIP_PDU * requ
                                                newTransport,
                                                request);
   if (!AddConnection(connection)) {
-    PTRACE(1, "SIP\tFailed to create SIPConnection for INVITE from " << request->GetURI() << " for " << toAddr);
+    PTRACE(1, "SIP\tFailed to create SIPConnection for INVITE for " << request->GetURI() << " to " << toAddr);
     request->SendResponse(transport, SIP_PDU::Failure_NotFound, this);
     return PFalse;
   }
 
-  m_receivedConnectionTokens.SetAt(mime.GetCallID(), connection->GetToken());
+  PString token = connection->GetToken();
+  m_receivedConnectionTokens.SetAt(mime.GetCallID(), token);
 
   // Get the connection to handle the rest of the INVITE in the thread pool
-  AddWork(new SIP_PDU_Work(*this, connection->GetToken(), request));
+  m_connectionThreadPool.AddWork(new SIP_PDU_Work(*this, token, request), token);
 
   return PTrue;
 }
 
-void SIPEndPoint::OnReceivedIntervalTooBrief(SIPTransaction & transaction, SIP_PDU & response)
-{
-  const SIPMIMEInfo & responseMIME = response.GetMIME();
-  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByCallID(responseMIME.GetCallID(), PSafeReadOnly);
-  if (handler == NULL)
-    return;
-
-  SIPTransaction *newTransaction = handler->CreateTransaction(transaction.GetTransport());
-  if (newTransaction) {
-    handler->SetExpire(responseMIME.GetMinExpires());
-    newTransaction->GetMIME().SetExpires(responseMIME.GetMinExpires());
-    newTransaction->GetMIME().SetCallID(handler->GetCallID());
-    if (newTransaction->Start())
-      return;
-  }
-
-  PTRACE(1, "SIP\t Could not restart REGISTER after IntervalTooBrief error!");
-  handler->OnFailed(SIP_PDU::Failure_IntervalTooBrief);
-  return;
-}
-
-void SIPEndPoint::OnReceivedAuthenticationRequired(SIPTransaction & transaction, SIP_PDU & response)
-{
-  // Try to find authentication information for the given call ID
-  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByCallID(response.GetMIME().GetCallID(), PSafeReadOnly);
-  if (handler == NULL)
-    return;
-  
-  handler->OnReceivedAuthenticationRequired(transaction, response);
-}
-
-
-void SIPEndPoint::OnReceivedOK(SIPTransaction & transaction, SIP_PDU & response)
-{
-  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByCallID(response.GetMIME().GetCallID(), PSafeReadOnly);
-  if (handler == NULL) 
-    return;
-  
-  handler->OnReceivedOK(transaction, response);
-}
-    
 
 void SIPEndPoint::OnTransactionFailed(SIPTransaction & transaction)
 {
-  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByCallID(transaction.GetMIME().GetCallID(), PSafeReadOnly);
-  if (handler == NULL) 
-    return;
-  
-  handler->OnTransactionFailed(transaction);
+  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByCallID(transaction.GetMIME().GetCallID(), PSafeReadWrite);
+  if (handler != NULL) 
+    handler->OnTransactionFailed(transaction);
+  else {
+    PTRACE(2, "SIP\tResponse received for unknown handler ID: " << transaction.GetMIME().GetCallID());
+  }
 }
 
 
@@ -905,14 +879,14 @@ PBoolean SIPEndPoint::OnReceivedNOTIFY(OpalTransport & transport, SIP_PDU & pdu)
   
   // A NOTIFY will have the same CallID than the SUBSCRIBE request it corresponds to
   // Technically should check for whole dialog, but call-id will do.
-  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByCallID(pdu.GetMIME().GetCallID(), PSafeReadOnly);
+  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByCallID(pdu.GetMIME().GetCallID(), PSafeReadWrite);
 
   if (handler == NULL && eventPackage == SIPSubscribe::MessageSummary) {
     PTRACE(4, "SIP\tWork around Asterisk bug in message-summary event package.");
     SIPURL url_from (pdu.GetMIME().GetFrom());
     SIPURL url_to (pdu.GetMIME().GetTo());
     PString to = url_to.GetUserName() + "@" + url_from.GetHostName();
-    handler = activeSIPHandlers.FindSIPHandlerByUrl(to, SIP_PDU::Method_SUBSCRIBE, eventPackage, PSafeReadOnly);
+    handler = activeSIPHandlers.FindSIPHandlerByUrl(to, SIP_PDU::Method_SUBSCRIBE, eventPackage, PSafeReadWrite);
   }
 
   if (handler == NULL) {
@@ -1075,17 +1049,12 @@ bool SIPEndPoint::Register(const SIPRegister::Params & params, PString & aor)
   else {
     // Otherwise create a new request with this method type
     handler = CreateRegisterHandler(params);
-    handler.SetSafetyMode(PSafeReadWrite);
     activeSIPHandlers.Append(handler);
   }
 
   aor = handler->GetAddressOfRecord().AsString();
 
-  if (handler->SendRequest(SIPHandler::Subscribing))
-    return true;
-
-  activeSIPHandlers.Remove(handler);
-  return false;
+  return handler->ActivateState(SIPHandler::Subscribing);
 }
 
 
@@ -1097,28 +1066,26 @@ SIPRegisterHandler * SIPEndPoint::CreateRegisterHandler(const SIPRegister::Param
 
 PBoolean SIPEndPoint::Unregister(const PString & addressOfRecord)
 {
-  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByUrl(addressOfRecord, SIP_PDU::Method_REGISTER, PSafeReadOnly);
-  if (handler == NULL) {
-    PTRACE(1, "SIP\tCould not find active REGISTER for " << addressOfRecord);
-    return false;
-  }
+  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByUrl(addressOfRecord, SIP_PDU::Method_REGISTER, PSafeReference);
+  if (handler != NULL)
+    return handler->ActivateState(SIPHandler::Unsubscribing);
 
-  return handler->SendRequest(SIPHandler::Unsubscribing);
+  PTRACE(1, "SIP\tCould not find active REGISTER for " << addressOfRecord);
+  return false;
 }
 
 
 bool SIPEndPoint::UnregisterAll()
 {
-  bool ok = true;
+  bool atLeastOne = false;
 
-  for (PSafePtr<SIPHandler> handler(activeSIPHandlers, PSafeReadOnly); handler != NULL; ++handler) {
-    if (handler->GetMethod() == SIP_PDU::Method_REGISTER) {
-      if (!handler->SendRequest(SIPHandler::Unsubscribing))
-        ok = false;
-    }
+  for (PSafePtr<SIPHandler> handler = activeSIPHandlers.GetFirstHandler(); handler != NULL; ++handler) {
+    if (handler->GetMethod() == SIP_PDU::Method_REGISTER &&
+        handler->ActivateState(SIPHandler::Unsubscribing))
+      atLeastOne = true;
   }
 
-  return ok;
+  return atLeastOne;
 }
 
 
@@ -1143,7 +1110,7 @@ bool SIPEndPoint::Subscribe(const SIPSubscribe::Params & params, PString & aor)
 
   // Create the SIPHandler structure
   PSafePtr<SIPSubscribeHandler> handler = PSafePtrCast<SIPHandler, SIPSubscribeHandler>(
-          activeSIPHandlers.FindSIPHandlerByUrl(params.m_addressOfRecord, SIP_PDU::Method_SUBSCRIBE, params.m_eventPackage, PSafeReadOnly));
+          activeSIPHandlers.FindSIPHandlerByUrl(params.m_addressOfRecord, SIP_PDU::Method_SUBSCRIBE, params.m_eventPackage, PSafeReadWrite));
   
   // If there is already a request with this URL and method, 
   // then update it with the new information
@@ -1157,7 +1124,7 @@ bool SIPEndPoint::Subscribe(const SIPSubscribe::Params & params, PString & aor)
 
   aor = handler->GetAddressOfRecord().AsString();
 
-  return handler->SendRequest(SIPHandler::Subscribing);
+  return handler->ActivateState(SIPHandler::Subscribing);
 }
 
 
@@ -1170,13 +1137,12 @@ bool SIPEndPoint::Unsubscribe(SIPSubscribe::PredefinedPackages eventPackage, con
 bool SIPEndPoint::Unsubscribe(const PString & eventPackage, const PString & to)
 {
   // Create the SIPHandler structure
-  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByUrl(to, SIP_PDU::Method_SUBSCRIBE, eventPackage, PSafeReadOnly);
-  if (handler == NULL) {
-    PTRACE(1, "SIP\tCould not find active SUBSCRIBE for " << to);
-    return PFalse;
-  }
+  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByUrl(to, SIP_PDU::Method_SUBSCRIBE, eventPackage, PSafeReference);
+  if (handler != NULL)
+    return handler->ActivateState(SIPHandler::Unsubscribing);
 
-  return handler->SendRequest(SIPHandler::Unsubscribing);
+  PTRACE(1, "SIP\tCould not find active SUBSCRIBE of " << eventPackage << " package to " << to);
+  return PFalse;
 }
 
 
@@ -1188,16 +1154,16 @@ bool SIPEndPoint::UnsubcribeAll(SIPSubscribe::PredefinedPackages eventPackage)
 
 bool SIPEndPoint::UnsubcribeAll(const PString & eventPackage)
 {
-  bool ok = true;
+  bool atLeastOne = false;
 
-  for (PSafePtr<SIPHandler> handler(activeSIPHandlers, PSafeReadOnly); handler != NULL; ++handler) {
-    if (handler->GetMethod() == SIP_PDU::Method_SUBSCRIBE && handler->GetEventPackage() == eventPackage) {
-      if (!handler->SendRequest(SIPHandler::Unsubscribing))
-        ok = false;
-    }
+  for (PSafePtr<SIPHandler> handler = activeSIPHandlers.GetFirstHandler(); handler != NULL; ++handler) {
+    if (handler->GetMethod() == SIP_PDU::Method_SUBSCRIBE &&
+        handler->GetEventPackage() == eventPackage &&
+        handler->ActivateState(SIPHandler::Unsubscribing))
+      atLeastOne = true;
   }
 
-  return ok;
+  return atLeastOne;
 }
 
 
@@ -1211,7 +1177,7 @@ bool SIPEndPoint::Notify(const SIPURL & aor, const PString & eventPackage, const
 {
   bool atLeastOne = false;
 
-  for (PSafePtr<SIPHandler> handler(activeSIPHandlers, PSafeReadWrite); handler != NULL; ++handler) {
+  for (PSafePtr<SIPHandler> handler = activeSIPHandlers.GetFirstHandler(); handler != NULL; ++handler) {
     if (handler->GetMethod() == SIP_PDU::Method_NOTIFY &&
         handler->GetAddressOfRecord() == aor &&
         handler->GetEventPackage() == eventPackage &&
@@ -1225,7 +1191,7 @@ bool SIPEndPoint::Notify(const SIPURL & aor, const PString & eventPackage, const
 PBoolean SIPEndPoint::Message(const PString & to, const PString & body)
 {
   // Create the SIPHandler structure
-  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByUrl(to, SIP_PDU::Method_MESSAGE, PSafeReadOnly);
+  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByUrl(to, SIP_PDU::Method_MESSAGE, PSafeReadWrite);
 
   // Otherwise create a new request with this method type
   if (handler != NULL)
@@ -1235,13 +1201,13 @@ PBoolean SIPEndPoint::Message(const PString & to, const PString & body)
     activeSIPHandlers.Append(handler);
   }
 
-  return handler->SendRequest(SIPHandler::Subscribing);
+  return handler->ActivateState(SIPHandler::Subscribing);
 }
 
 PBoolean SIPEndPoint::Message(const PString & to, const PString & body, const PString & remoteContact, const PString & callId)
 {
   // Create the SIPHandler structure
-  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByCallID(callId, PSafeReadOnly);
+  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByCallID(callId, PSafeReadWrite);
   
   // Otherwise create a new request with this method type
   if (handler != NULL)
@@ -1251,7 +1217,7 @@ PBoolean SIPEndPoint::Message(const PString & to, const PString & body, const PS
     activeSIPHandlers.Append(handler);
   }
 
-  return handler->SendRequest(SIPHandler::Subscribing);
+  return handler->ActivateState(SIPHandler::Subscribing);
 }
 
 
@@ -1272,11 +1238,11 @@ void SIPEndPoint::OnMessageReceived (const SIPURL & /*from*/, const PString & /*
 
 PBoolean SIPEndPoint::Ping(const PString & to)
 {
-  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByUrl(to, SIP_PDU::Method_PING, PSafeReadOnly);
+  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByUrl(to, SIP_PDU::Method_PING, PSafeReference);
   if (handler == NULL)
     handler = new SIPPingHandler(*this, to);
 
-  return handler->SendRequest(SIPHandler::Subscribing);
+  return handler->ActivateState(SIPHandler::Subscribing);
 }
 
 
@@ -1292,7 +1258,7 @@ bool SIPEndPoint::Publish(const SIPSubscribe::Params & params, const PString & b
 
   aor = handler->GetAddressOfRecord().AsString();
 
-  return handler->SendRequest(params.m_expire != 0 ? SIPHandler::Subscribing : SIPHandler::Unsubscribing);
+  return handler->ActivateState(params.m_expire != 0 ? SIPHandler::Subscribing : SIPHandler::Unsubscribing);
 }
 
 
@@ -1406,8 +1372,8 @@ unsigned SIPEndPoint::GetAllowedMethods() const
 
 PBoolean SIPEndPoint::GetAuthentication(const PString & authRealm, PString & realm, PString & user, PString & password) 
 {
-  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByAuthRealm(authRealm, PString::Empty(), PSafeReadOnly);
-  if (handler == NULL)
+  PSafePtr<SIPHandler> handler = activeSIPHandlers.FindSIPHandlerByAuthRealm(authRealm, user, PSafeReadOnly);
+  if (handler == NULL || handler->GetPassword ().IsEmpty ())
     return PFalse;
 
   realm    = handler->GetRealm();
@@ -1563,8 +1529,13 @@ void SIPEndPoint::SIP_PDU_Thread::AddWork(SIP_PDU_Work * work)
 }
 
 
-void SIPEndPoint::SIP_PDU_Thread::RemoveWork(SIP_PDU_Work *)
+void SIPEndPoint::SIP_PDU_Thread::RemoveWork(SIP_PDU_Work * work)
 {
+  m_workerMutex.Wait();
+  m_pduQueue.pop();
+  m_workerMutex.Signal();
+
+  delete work;
 }
 
 
@@ -1579,29 +1550,22 @@ void SIPEndPoint::SIP_PDU_Thread::Main()
 {
   while (!m_shutdown) {
 
-    // wait for work to become available
+    // get the work
     m_workerMutex.Wait();
-    if (m_pduQueue.size() == 0) {
-      m_workerMutex.Signal();
+    SIP_PDU_Work * work = m_pduQueue.size() > 0 ? m_pduQueue.front() : NULL;
+    m_workerMutex.Signal();
+
+    // wait for work to become available
+    if (work == NULL) {
       m_sync.Wait();
       continue;
     }
 
-    // get the work
-    SIP_PDU_Work * work = m_pduQueue.front();
-    m_pduQueue.pop();
-    m_workerMutex.Signal();
-
     // process the work
-    PTRACE(2, "SIP\tStarted processing PDU");
     work->OnReceivedPDU();
-    PTRACE(2, "SIP\tFinished processing PDU");
 
     // indicate work is now free
-    m_pool.RemoveWork(work, false);
-
-    // delete the work
-    delete work;
+    m_pool.RemoveWork(work);
   }
 }
 
@@ -1610,6 +1574,8 @@ SIPEndPoint::SIP_PDU_Work::SIP_PDU_Work(SIPEndPoint & ep, const PString & token,
   , m_token(token)
   , m_pdu(pdu)
 {
+  PTRACE(4, "SIP\tQueueing PDU \"" << *m_pdu << "\", transaction="
+         << m_pdu->GetTransactionID() << ", token=" << m_token);
 }
 
 
@@ -1633,10 +1599,9 @@ void SIPEndPoint::SIP_PDU_Work::OnReceivedPDU()
     else {
       PTRACE(2, "SIP\tCannot find transaction " << transactionID << " for response PDU \"" << *m_pdu << '"');
     }
-    return;
   }
 
-  if (PAssert(!m_token.IsEmpty(), PInvalidParameter)) {
+  else if (PAssert(!m_token.IsEmpty(), PInvalidParameter)) {
     PTRACE(4, "SIP\tHandling PDU \"" << *m_pdu << "\" for token=" << m_token);
     PSafePtr<SIPConnection> connection = m_endpoint.GetSIPConnectionWithLock(m_token, PSafeReference);
     if (connection != NULL) 
@@ -1645,6 +1610,8 @@ void SIPEndPoint::SIP_PDU_Work::OnReceivedPDU()
       PTRACE(2, "SIP\tCannot find connection for PDU \"" << *m_pdu << "\" using token=" << m_token);
     }
   }
+
+  PTRACE(4, "SIP\tHandled PDU \"" << *m_pdu << '"');
 }
 
 
@@ -1658,9 +1625,9 @@ SIPEndPoint::InterfaceMonitor::InterfaceMonitor(SIPEndPoint & ep, PINDEX priorit
 void SIPEndPoint::InterfaceMonitor::OnAddInterface(const PIPSocket::InterfaceEntry &)
 {
   if (priority == SIPEndPoint::LowPriority) {
-    for (PSafePtr<SIPHandler> handler(m_endpoint.activeSIPHandlers, PSafeReadOnly); handler != NULL; ++handler) {
+    for (PSafePtr<SIPHandler> handler = m_endpoint.activeSIPHandlers.GetFirstHandler(); handler != NULL; ++handler) {
       if (handler->GetState() == SIPHandler::Unavailable)
-        handler->SendRequest(SIPHandler::Restoring);
+        handler->ActivateState(SIPHandler::Restoring);
     }
   } else {
     // special case if interface filtering is used: A new interface may 'hide' the old interface.
@@ -1671,7 +1638,7 @@ void SIPEndPoint::InterfaceMonitor::OnAddInterface(const PIPSocket::InterfaceEnt
     // the transport will still listen on the old interface only. Therefore, clear the
     // socket binding BEFORE the monitored sockets update their interfaces.
     if (PInterfaceMonitor::GetInstance().HasInterfaceFilter()) {
-      for (PSafePtr<SIPHandler> handler(m_endpoint.activeSIPHandlers, PSafeReadOnly); handler != NULL; ++handler) {
+      for (PSafePtr<SIPHandler> handler = m_endpoint.activeSIPHandlers.GetFirstHandler(PSafeReadOnly); handler != NULL; ++handler) {
         OpalTransport *transport = handler->GetTransport();
         if (transport != NULL) {
           PString iface = transport->GetInterface();
@@ -1698,12 +1665,12 @@ void SIPEndPoint::InterfaceMonitor::OnAddInterface(const PIPSocket::InterfaceEnt
 void SIPEndPoint::InterfaceMonitor::OnRemoveInterface(const PIPSocket::InterfaceEntry & entry)
 {
   if (priority == SIPEndPoint::LowPriority) {
-    for (PSafePtr<SIPHandler> handler(m_endpoint.activeSIPHandlers, PSafeReadOnly); handler != NULL; ++handler) {
+    for (PSafePtr<SIPHandler> handler = m_endpoint.activeSIPHandlers.GetFirstHandler(PSafeReadOnly); handler != NULL; ++handler) {
       if (handler->GetState() == SIPHandler::Subscribed &&
           handler->GetTransport() != NULL &&
           handler->GetTransport()->GetInterface().Find(entry.GetName()) != P_MAX_INDEX) {
         handler->GetTransport()->SetInterface(PString::Empty());
-        handler->SendRequest(SIPHandler::Refreshing);
+        handler->ActivateState(SIPHandler::Refreshing);
       }
     }
   }
