@@ -23,9 +23,9 @@
  *
  * Contributor(s): ______________________________________.
  *
- * $Revision: 23493 $
- * $Author: rjongbloed $
- * $Date: 2009-09-19 02:00:38 +0000 (Sat, 19 Sep 2009) $
+ * $Revision: 24330 $
+ * $Author: ededu $
+ * $Date: 2010-05-07 13:34:54 -0500 (Fri, 07 May 2010) $
  */
 
 #include <ptlib.h>
@@ -1460,12 +1460,18 @@ PObject::Comparison SIPDigestAuthentication::Compare(const PObject & other) cons
   if (otherAuth == NULL)
     return LessThan;
 
-  Comparison result = GetAuthRealm().Compare(otherAuth->GetAuthRealm());
+  if (algorithm < otherAuth->algorithm)
+    return LessThan;
+  if (algorithm > otherAuth->algorithm)
+    return GreaterThan;
+
+  Comparison result = authRealm.Compare(otherAuth->authRealm);
   if (result != EqualTo)
     return result;
 
-  if (GetAlgorithm() != otherAuth->GetAlgorithm())
-    return GetAlgorithm() < otherAuth->GetAlgorithm() ? LessThan : GreaterThan;
+  result = nonce.Compare(otherAuth->nonce);
+  if (result != EqualTo)
+    return result;
 
   return SIPAuthentication::Compare(other);
 }
@@ -1533,7 +1539,7 @@ PBoolean SIPDigestAuthentication::Parse(const PString & _auth, PBoolean proxy)
 
 PBoolean SIPDigestAuthentication::Authorise(SIP_PDU & pdu) const
 {
-  PTRACE(3, "SIP\tAdding authentication information");
+  PTRACE(3, "SIP\tAdding authentication information for user \"" << username << "\" at realm \"" << authRealm << '"');
 
   PMessageDigest5 digestor;
   PMessageDigest5::Code a1, a2, entityBodyCode, response;
@@ -2019,22 +2025,26 @@ bool SIP_PDU::SendResponse(OpalTransport & transport, SIP_PDU & response, SIPEnd
     PString proto = viaList.front();
     PString viaPort = defaultPort;
 
-    PINDEX j = 0;
+    PINDEX bracket = viaAddress.FindLast(']');
+    if (bracket == P_MAX_INDEX)
+      bracket = 0;
+
+    PINDEX pos = 0;
     // get the address specified in the Via
-    if ((j = viaAddress.FindLast (' ')) != P_MAX_INDEX)
-      viaAddress = viaAddress.Mid(j+1);
-    if ((j = viaAddress.Find (';')) != P_MAX_INDEX)
-      viaAddress = viaAddress.Left(j);
-    if ((j = viaAddress.Find (':')) != P_MAX_INDEX) {
-      viaPort = viaAddress.Mid(j+1);
-      viaAddress = viaAddress.Left(j);
+    if ((pos = viaAddress.FindLast (' ')) != P_MAX_INDEX)
+      viaAddress = viaAddress.Mid(pos+1);
+    if ((pos = viaAddress.Find (';')) != P_MAX_INDEX)
+      viaAddress = viaAddress.Left(pos);
+    if ((pos = viaAddress.Find (':')) != P_MAX_INDEX && (pos > bracket)) {
+      viaPort = viaAddress.Mid(pos+1);
+      viaAddress = viaAddress.Left(pos);
     }
 
     // get the protocol type from Via header
-    if ((j = proto.FindLast (' ')) != P_MAX_INDEX)
-      proto = proto.Left(j);
-    if ((j = proto.FindLast('/')) != P_MAX_INDEX)
-      proto = proto.Mid(j+1);
+    if ((pos = proto.FindLast (' ')) != P_MAX_INDEX)
+      proto = proto.Left(pos);
+    if ((pos = proto.FindLast('/')) != P_MAX_INDEX)
+      proto = proto.Mid(pos+1);
 
     // maddr is present, no support for multicast yet
     PString param = SIPMIMEInfo::ExtractFieldParameter(viaList.front(), "maddr");
