@@ -23,9 +23,9 @@
  *
  * Contributor(s): Vyacheslav Frolov.
  *
- * $Revision: 23165 $
+ * $Revision: 23782 $
  * $Author: rjongbloed $
- * $Date: 2009-07-29 04:57:29 +0000 (Wed, 29 Jul 2009) $
+ * $Date: 2009-11-19 23:59:13 -0600 (Thu, 19 Nov 2009) $
  */
 
 #include <ptlib.h>
@@ -347,6 +347,8 @@ PBoolean OpalFaxMediaStream::ReadPacket(RTP_DataFrame & packet)
     PINDEX len = m_faxCallInfo->socket.GetLastReadCount();
     packet.SetPayloadType(RTP_DataFrame::MaxPayloadType);
     packet.SetPayloadSize(len);
+    packet.SetTimestamp(timestamp);
+    timestamp += len/2;
 
 #if WRITE_PCM_FILE
     static int file = _open("t38_audio_in.pcm", _O_BINARY | _O_CREAT | _O_TRUNC | _O_WRONLY, _S_IREAD | _S_IWRITE);
@@ -471,11 +473,6 @@ PBoolean OpalFaxMediaStream::Close()
 
     PWaitAndSignal m2(faxMapMutex);
 
-    if (faxCallInfo->stdoutThread != NULL) {
-      delete faxCallInfo->stdoutThread;
-      faxCallInfo->stdoutThread = NULL;
-    }
-
     OpalFaxCallInfoMap_T::iterator r = faxCallInfoMap.find(sessionToken);
     if (r == faxCallInfoMap.end()) {
       PTRACE(1, "Fax\tError: media stream not found in T38 session list");
@@ -501,6 +498,11 @@ PBoolean OpalFaxMediaStream::Close()
     faxCallInfoMap.erase(sessionToken);
   }
 
+  if (faxCallInfo->stdoutThread != NULL) {
+    delete faxCallInfo->stdoutThread;
+    faxCallInfo->stdoutThread = NULL;
+  }
+
   // delete the object
   delete faxCallInfo;
 
